﻿#Region " Imports "
Imports Grid.Settings
Imports Grid
Imports Data.Tsql
Imports Data

Imports Infragistics.Web.UI.GridControls
Imports Infragistics.Web.UI

#End Region

Public Class WebTabForm
    Inherits System.Web.UI.Page

#Region " Declarations "
    Private _dataSet As New DataSet
    Private _demoData As New Data.Database
    Private WithEvents _gridSettings As New Grid.Settings
    Private WithEvents _pager As New GridPagerUserControl
    Private _fileName As String = String.Format("{0} {1:d}", "Export", Date.Now())
#End Region

#Region " Constants "
    Private Const TABLE_NAME As String = "Parent"
    Private Const RECORDS_PER_PAGE As Integer = 5
    Private Const HIDDEN_FIELDNAME_PREFIX As String = "Hidden "
    Private Const WORKSHEET_NAME As String = "Export"
    Private Const DEFAULT_VIEW_INDEX As Integer = 0
#End Region

#Region " Page Events "
    Private Sub Page_Init(sender As Object, e As System.EventArgs) Handles Me.Init

        Me._pager = TryCast(Me.WebTab1.Tabs(0).FindControl("GridPager"), GridPagerUserControl)
        AddHandler Me._pager.PageChanged, AddressOf Pager_PageChanged
        AddHandler Me._pager.ExportedToExcel, AddressOf Pager_ExportToExcel
        AddHandler Me._pager.NumberOfRowsChanged, AddressOf Pager_NumberOfRowsChanged
        AddHandler Me._pager.ViewChanged, AddressOf Pager_ViewChanged

    End Sub

    Protected Sub Page_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load

        If Not (IsPostBack()) Then
            Initialize_GridSettings()
            Initialize_GridView(DEFAULT_VIEW_INDEX, Me._gridSettings)
            Initialize_Grid()

            Me.BindGrid()
        End If

    End Sub

    Protected Sub Page_UnLoad(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Unload
        If Not (Me._gridSettings Is Nothing) Then
            Me._gridSettings.Dispose()
        End If
    End Sub
#End Region

#Region " Events "
    Protected Sub whdg_InitializeRow(sender As Object, e As Infragistics.Web.UI.GridControls.RowEventArgs) Handles whdgGrid.InitializeRow
        If Grid.Settings.IsRowOnCurrentGridPage(Me._gridSettings, e.Row.Index) Then
            Grid.Settings.Initialize_HierarchicalGridRow(e)
        End If
    End Sub

    Public Sub whdg_ColumnMoved(ByVal sender As Object, e As Infragistics.Web.UI.GridControls.ColumnMovingEventArgs) Handles whdgGrid.ColumnMoved
        If e.Column Is Nothing Then
            Return
        End If

        Dim columnMoved As New GridColumn(e.Column)
        Dim gridColumns As New GridColumnCollection(Me.whdgGrid.GridView.Columns)

        gridColumns.RemoveAt(e.PreviousVisibleIndex)
        gridColumns.Insert(e.Column.VisibleIndex, columnMoved)
        _gridSettings.GridColumns = gridColumns

        Me.BindGrid()

    End Sub

    Protected Sub whdg_ColumnResized(ByVal sender As Object, e As Infragistics.Web.UI.GridControls.ColumnResizingEventArgs) Handles whdgGrid.ColumnResized

        Me.BindGrid()

    End Sub

    Protected Sub whdg_ColumnSorted(ByVal sender As Object, e As Infragistics.Web.UI.GridControls.SortingEventArgs) Handles whdgGrid.ColumnSorted

        Me.BindGrid()

    End Sub

    Protected Sub whdg_GroupedRowInitialized(sender As Object, e As Infragistics.Web.UI.GridControls.GroupedRowEventArgs) Handles whdgGrid.GroupedRowInitialized
        e.GroupedRow.Text = HttpUtility.UrlDecode(e.GroupedRow.Text)
    End Sub

    Protected Sub whdg_GroupedColumnsChanging(sender As Object, e As Infragistics.Web.UI.GridControls.GroupedColumnsChangingEventArgs) Handles whdgGrid.GroupedColumnsChanging
        Dim grpColCollection As GridGroupedColumnCollection
        Dim groupCriteria As New GroupByCollection(_gridSettings.GroupByString)

        Select Case e.Action
            Case GroupByChangeAction.Group
                If Not e.GroupedColumns.Contains(e.EffectedColumns(0)) Then
                    If groupCriteria.Count > 0 Then
                        For Each column As GroupBy In groupCriteria
                            If Not (column.ColumnName = e.EffectedColumns(0).ColumnKey) Then
                                e.GroupedColumns.Add(e.EffectedColumns(0))
                            End If
                        Next
                    Else
                        e.GroupedColumns.Add(e.EffectedColumns(0))
                    End If

                    grpColCollection = New GridGroupedColumnCollection(e.GroupedColumns)
                    Me._gridSettings.GroupedColumnSettings = grpColCollection
                    Me.BindGrid()
                    Me.SetPagerContext(Me._pager, Me._gridSettings, Grid.GridType.TypeOne)
                End If
            Case GroupByChangeAction.Ungroup
                If e.GroupedColumns.Contains(e.EffectedColumns(0)) Then
                    If groupCriteria.Count > 0 Then
                        For Each column As GroupBy In groupCriteria
                            If (column.ColumnName = e.EffectedColumns(0).ColumnKey) Then
                                e.GroupedColumns.Remove(e.EffectedColumns(0))
                            End If
                        Next
                    Else
                        If e.GroupedColumns.Count > 1 Then
                            e.GroupedColumns.Remove(e.EffectedColumns(0))
                            grpColCollection = New GridGroupedColumnCollection(e.GroupedColumns)
                            Me._gridSettings.GroupedColumnSettings = grpColCollection
                        Else
                            e.GroupedColumns.Clear()
                            grpColCollection = New GridGroupedColumnCollection(e.GroupedColumns)
                            Me._gridSettings.GroupedColumnSettings = grpColCollection
                        End If
                    End If
                    Me.BindGrid()
                    Me.SetPagerContext(Me._pager, Me._gridSettings, Grid.GridType.TypeOne)
                End If
            Case GroupByChangeAction.Move
                grpColCollection = New GridGroupedColumnCollection(e.GroupedColumns)
                Me._gridSettings.GroupedColumnSettings = grpColCollection
                Me.BindGrid()
                Me.SetPagerContext(Me._pager, Me._gridSettings, Grid.GridType.TypeOne)
            Case GroupByChangeAction.ChangeSortDirection
                grpColCollection = New GridGroupedColumnCollection(e.GroupedColumns)
                Me._gridSettings.GroupedColumnSettings = grpColCollection
                Me.BindGrid()
                Me.SetPagerContext(Me._pager, Me._gridSettings, Grid.GridType.TypeOne)
        End Select
    End Sub

    Private Sub Pager_ViewChanged(sender As Object, e As GridPagerViewChangedEventArgs)

        Me._gridSettings.SelectedViewIndex = e.SelectedViewIndex
        Me._gridSettings.SelectedViewName = e.SelectedViewName

        Me._gridSettings.PageIndex = 0
        Me._demoData.SelectedValue = e.SelectedViewIndex

        Initialize_GridView(e.SelectedViewIndex, Me._gridSettings)
        Initialize_Grid()

        Me.BindGrid()
    End Sub

    Private Sub Pager_PageChanged(sender As Object, e As GridPagerSettingsChangedEventArgs)

        If (e.TotalNumberOfPages - 1) < e.PageIndex Then
            _gridSettings.PageIndex = 0
        Else
            _gridSettings.PageIndex = e.PageIndex
        End If

        Me.BindGrid()

    End Sub

    Private Sub Pager_NumberOfRowsChanged(sender As Object, e As GridPagerPageSizeChangedEventArgs)

        Me._gridSettings.PageSize = e.NewPageSize
        Me._gridSettings.PageIndex = 0

        Me.BindGrid()

    End Sub

    Private Sub Pager_ExportToExcel(sender As Object, e As GridPagerExportedToExcelEventArgs)
        If e.ExportToExcelFlag Then
            Me.BindGrid()
            Me._pager.ExportGridToExcel(Me.whdgGrid)
        End If
    End Sub
#End Region

#Region " Methods "
    Private Sub Initialize_DemoData()
        If Me._dataSet.Tables.Contains(TABLE_NAME) Then
            Me._dataSet.Tables.Remove(TABLE_NAME)
        End If
        Me._dataSet.Tables.Add(Me._demoData.BuildDataTable(TABLE_NAME))
    End Sub

    Protected Sub Initialize_GridView(ByVal iSelectedViewIndex As Integer, ByRef dgs As Grid.Settings)
        If Not (dgs.GroupedColumnSettings Is Nothing) Then
            dgs.GroupedColumnSettings = New GridGroupedColumnCollection()
        End If

        If Not (dgs.GridColumns Is Nothing) Then
            'dgs.GridColumns = New GridColumnCollection()
        End If

        Grid.Settings.InitializeGridView(iSelectedViewIndex, dgs)

    End Sub

    Protected Sub Initialize_GridFilter(ByRef dgs As Grid.Settings)
        If dgs.WhereString = String.Empty Then
            dgs.WhereString = String.Empty
        Else
            dgs.WhereString = dgs.WhereString & String.Empty
        End If
    End Sub

    Protected Sub Initialize_GridOrderByFilter(ByRef dgs As Grid.Settings)
        dgs.OrderByString = String.Empty
    End Sub

    Private Sub Initialize_GridSettings()
        _gridSettings = New Grid.Settings(Me.whdgGrid.ClientID, Grid.GridType.TypeOne, 1, 0, RECORDS_PER_PAGE)
        Dim grpColCollection As New GridGroupedColumnCollection(Me.whdgGrid.GroupingSettings.GroupedColumns)

        With _gridSettings
            .GroupedColumnSettings = grpColCollection
            .SelectedViewName = "View 1"
            .PageSize = RECORDS_PER_PAGE
        End With
    End Sub

    Private Sub Initialize_Grid()

        Grid.Settings.Initialize_HierarchicalGridGrouping(Me.whdgGrid, _gridSettings)
        Grid.Settings.Initialize_HierarchicalGridColumns(Me.whdgGrid, _gridSettings, Grid.GridType.TypeOne)

        Initialize_GridFilter(_gridSettings)
        Initialize_GridOrderByFilter(_gridSettings)

    End Sub

    Private Sub BindGrid()
        Me.Initialize_DemoData()

        'Me.whdgGrid.GridView.ClearDataSource()
        Me.whdgGrid.DataSource = Me._dataSet
        Me.whdgGrid.DataKeyFields = Data.FieldName.HiddenRef
        Me.whdgGrid.Key = "Test Data"

        'Me.whdgGrid.DataBind()

        Me._gridSettings.RecordCount = _demoData.RecordCount

        Me.SetPagerContext(Me._pager, Me._gridSettings, Grid.GridType.TypeOne)
        Me.SetGridContext(Me.whdgGrid.GridView, Me._gridSettings)

    End Sub

    Private Shared Function GetClientIncludes() As String
        'Return String.Format( _
        '"<script type=""text/javascript"" src=""{0}""></script>", "scripts/ClientSide.js")
        Return String.Empty
    End Function

    Private Sub SetPagerContext(ByRef pager As GridPagerUserControl, ByVal dgs As Grid.Settings, ByVal gridType As GridType)
        pager.SetContext(dgs.RecordCount, dgs.PageSize, _fileName, WORKSHEET_NAME, _
                         DataExportMode.AllDataInDataSource, dgs.RecordCount, _
                         dgs.SelectedViewIndex, dgs.SelectedViewName, gridType)
    End Sub

    Private Sub SetGridContext(ByRef grid As GridControls.ContainerGrid, ByVal dgs As Grid.Settings)
        grid.Behaviors.Paging.PageIndex = dgs.PageIndex
        grid.Behaviors.Paging.PageSize = dgs.PageSize
    End Sub
#End Region

#Region " Overrides "
    Protected Overrides Sub OnPreRender(ByVal e As System.EventArgs)
        MyBase.OnPreRender(e)

        ' Register client script
        Dim strScriptKey As String = Me.GetType().ToString()

        If Not (ClientScript.IsClientScriptBlockRegistered(strScriptKey)) Then
            ClientScript.RegisterClientScriptBlock(GetType(Page), strScriptKey, GetClientIncludes())
        End If

        ' Register startup script
        If Not (ClientScript.IsStartupScriptRegistered(strScriptKey)) Then
            Dim sbScript As New StringBuilder("<script type=""text/javascript"">")
            sbScript.Append(vbNewLine)
            sbScript.Append("<!--")
            sbScript.Append(vbNewLine)
            sbScript.AppendFormat("var g_Whdg = {0};", Me.whdgGrid.ClientID)
            sbScript.Append(vbNewLine)
            sbScript.AppendFormat("var g_Tab = {0};", Me.WebTab1.ClientID)
            sbScript.Append(vbNewLine)
            sbScript.AppendFormat("-->")
            sbScript.Append(vbNewLine)
            sbScript.AppendFormat("</script>")
            ClientScript.RegisterStartupScript(GetType(Page), strScriptKey, sbScript.ToString)
        End If

    End Sub

    ''' <summary>
    ''' Restores view-state information from a previous page request that was saved 
    ''' by the <c>SaveViewState</c> method.
    ''' </summary>
    Protected Overrides Sub LoadViewState(ByVal savedState As Object)
        MyBase.LoadViewState(savedState)
        _gridSettings = CType(ViewState("gridSettings"), Grid.Settings)
    End Sub

    ''' <summary>
    ''' Saves any page view-state changes that have occurred since the time the page 
    ''' was posted back to the server.
    ''' </summary>
    Protected Overrides Function SaveViewState() As Object
        If Not (_gridSettings Is Nothing) Then
            ViewState("gridSettings") = _gridSettings
        End If

        Return MyBase.SaveViewState

    End Function
#End Region
End Class